//
//  MoneyField.m
//
//  Created by John Schember on 2/16/16.
//  Copyright © 2016 Monetra Technologies, LLC All rights reserved.
//

/* This can only use up to 9 digits (N 7.2) because the money formatter needs
 * to convert to a double limiting the range.
 *
 * Will stop adding digits once the maximum has been reached. Programmatically
 * settings more will result in truncation.
 *
 * When setting programmatically non-numerics will be stripped and numerics will be used.
 */

#import <Foundation/Foundation.h>
#import "MoneyField.h"

/* Takes a string and turns it into X.XX with numerics. 
 * Will strip non-numerics.
 */
static NSString *formatMoney(NSString *money) {
    NSString *numstr = [[money componentsSeparatedByCharactersInSet:[[NSCharacterSet decimalDigitCharacterSet] invertedSet]] componentsJoinedByString:@""];
    
    NSMutableString *mystr = [NSMutableString stringWithString:numstr];
    if (mystr.length > 9)
        [mystr deleteCharactersInRange:NSMakeRange(9, mystr.length-9)];
    
    int64_t val = [mystr intValue];
    return [NSString stringWithFormat:@"%lld.%02lld", val/100, val%100];
}

/* Delegate to handle internal delegate events but allow forwarding
 * to an outside delegate for events we don't care about. */
@interface MoneyFieldDelegate : NSObject <UITextFieldDelegate>

@property (weak, nonatomic) id<UITextFieldDelegate> delegate;

@end

@implementation MoneyFieldDelegate

/* Check our and the delegates selectors and determine if either implements them. */
- (BOOL)respondsToSelector:(SEL)aSelector {
    if ([super respondsToSelector:aSelector])
        return YES;
    return [_delegate respondsToSelector:aSelector];
}

/* We didn't capture the selector so forward it along to the outside delegate. */
- (id)forwardingTargetForSelector:(SEL)aSelector {
    return _delegate;
}

/* This is called when text in the field is modified. */
- (BOOL)textField:(UITextField*)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString*)string {
    
    NSMutableString *mystr = [NSMutableString stringWithString:textField.text];
    
    /* backspace if no text added, or add the text and set it. */
    if (string.length == 0) {
        [mystr deleteCharactersInRange:NSMakeRange(mystr.length-1, 1)];
    } else {
        [mystr appendString:string];
    }
    
    [textField setText:mystr];
    return NO;
}

@end

@implementation MoneyField

/* Formatter makes the number look pretty by doing things
 * like puttings $ in front. */
NSNumberFormatter *_formatter;
MoneyFieldDelegate *_money_delegate;

/* Inits for various situations. Internal one that always
 * gets called to setup the sub class variables. */
- (void)int_init {
    _formatter = [[NSNumberFormatter alloc] init];
    _formatter.locale = [NSLocale currentLocale];
    _formatter.numberStyle = NSNumberFormatterCurrencyStyle;
    _formatter.currencyCode = @"USD";
    _formatter.usesGroupingSeparator = YES;
    _formatter.maximumFractionDigits = 2;
    _formatter.minimumFractionDigits = 2;
    _formatter.maximumIntegerDigits = 7;
    
    _money_delegate = [[MoneyFieldDelegate alloc] init];
    [super setDelegate:_money_delegate];

    [self setKeyboardType:UIKeyboardTypeNumberPad];
    [self setSpellCheckingType:UITextSpellCheckingTypeNo];
    [self setAutocapitalizationType:UITextAutocapitalizationTypeNone];
    [self setAutocorrectionType:UITextAutocorrectionTypeNo];
    
    self.textAlignment = NSTextAlignmentRight;
    
    [self setText:@"0"];
}

- (id)init {
    self = [super init];
    if (!self)
        return nil;
    [self int_init];
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (!self)
        return nil;
    [self int_init];
    return self;
}

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (!self)
        return nil;
    [self int_init];
    return self;
}

#if 0
/* Don't show the caret. */
- (CGRect)caretRectForPosition:(UITextPosition *)position {
    return CGRectZero;
}
#endif

/* Don't allow text selection. */
- (NSArray *)selectionRectsForRange:(UITextRange *)range {
    return nil;
}

/* Don't allow selection menu actions. */
- (BOOL)canPerformAction:(SEL)action withSender:(id)sender {
    if (action == @selector(cut:) || action == @selector(copy) || action == @selector(paste:) || action == @selector(select:) || action == @selector(selectAll:))
        return NO;
    return [super canPerformAction:action withSender:sender];
}

/* Store the delegate inside the internal delegate. */
- (void) setDelegate:(id<UITextFieldDelegate>)delegate {
    _money_delegate.delegate = delegate;
}

- (id<UITextFieldDelegate>)delegate {
    return _money_delegate.delegate;
}

/* Take any text input and turn it into a proper formated money string. */
- (void)setText:(NSString *)text {
    [super setText:[_formatter stringFromNumber:[NSNumber numberWithDouble:[formatMoney(text) doubleValue]]]];
}

/* Take the input and return it as X.XX. This will strip things like $ at the front. */
- (NSString *)text {
    return formatMoney([super text]);
}

@end
